@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2007-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -23,7 +23,7 @@ from mailman.app.notifications import (
     send_welcome_message)
 from mailman.core.i18n import _
 from mailman.email.message import OwnerNotification
-from mailman.interfaces.address import IAddress
+from mailman.interfaces.address import IAddress, InvalidEmailAddressError
 from mailman.interfaces.bans import IBanManager
 from mailman.interfaces.member import (
     AlreadySubscribedError, MemberRole, MembershipIsBannedError,
@@ -53,12 +53,17 @@ def add_member(mlist, record, role=MemberRole.member):
     :rtype: `IMember`
     :raises AlreadySubscribedError: if the user is already subscribed to
         the mailing list.
-    :raises InvalidEmailAddressError: if the email address is not valid.
+    :raises InvalidEmailAddressError: if the email address is not valid or
+        is the list posting address.
     :raises MembershipIsBannedError: if the membership is not allowed.
     """
     # Check to see if the email address is banned.
     if IBanManager(mlist).is_banned(record.email):
         raise MembershipIsBannedError(mlist, record.email)
+    # Check for list posting address, but allow for nonmember.
+    if (record.email.lower() == mlist.posting_address and
+            role != MemberRole.nonmember):
+        raise InvalidEmailAddressError('List posting address not allowed')
     # Make sure there is a user linked with the given address.
     user_manager = getUtility(IUserManager)
     user = user_manager.make_user(record.email, record.display_name)
@@ -164,6 +169,8 @@ def handle_SubscriptionEvent(event):
             address = subscriber.preferred_address.email
             display_name = subscriber.display_name
         send_admin_subscription_notice(mlist, address, display_name)
-    # Maybe send a welcome message to the new member.
-    if mlist.send_welcome_message:
+    # Maybe send a welcome message to the new member. The event's flag
+    # overrides the mailinglist's configuration, iff it is non-None.
+    if ((event.send_welcome_message is None and mlist.send_welcome_message)
+            or event.send_welcome_message):
         send_welcome_message(mlist, member, member.preferred_language)
