@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2007-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -20,6 +20,7 @@
 import time
 import logging
 
+from dateutil.tz import tzlocal
 from email.utils import formatdate, getaddresses, make_msgid
 from mailman.app.membership import delete_member
 from mailman.config import config
@@ -134,7 +135,7 @@ def handle_message(mlist, id, action, comment=None, forward=None):
         # here can lead to a huge delay in web turnaround.  Log the moderation
         # and add a header.
         msg['X-Mailman-Approved-At'] = formatdate(
-            time.mktime(now().timetuple()), localtime=True)
+            time.mktime(now(tz=tzlocal()).timetuple()), localtime=True)
         vlog.info('held message approved, message-id: %s',
                   msg.get('message-id', 'n/a'))
         # Stick the message back in the incoming queue for further
