@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2009-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -28,7 +28,7 @@ from mailman.app.workflow import Workflow
 from mailman.core.i18n import _
 from mailman.database.transaction import flush
 from mailman.email.message import UserNotification
-from mailman.interfaces.address import IAddress
+from mailman.interfaces.address import IAddress, InvalidEmailAddressError
 from mailman.interfaces.bans import IBanManager
 from mailman.interfaces.listmanager import ListDeletingEvent
 from mailman.interfaces.mailinglist import SubscriptionPolicy
@@ -38,7 +38,8 @@ from mailman.interfaces.member import (
 from mailman.interfaces.pending import IPendable, IPendings
 from mailman.interfaces.subscriptions import (
     ISubscriptionManager, ISubscriptionService,
-    SubscriptionConfirmationNeededEvent, SubscriptionPendingError, TokenOwner,
+    SubscriptionConfirmationNeededEvent, SubscriptionInvitationNeededEvent,
+    SubscriptionPendingError, TokenOwner,
     UnsubscriptionConfirmationNeededEvent)
 from mailman.interfaces.template import ITemplateLoader
 from mailman.interfaces.user import IUser
@@ -167,18 +168,27 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         'pre_approved',
         'pre_confirmed',
         'pre_verified',
+        'invitation',
         'address_key',
         'subscriber_key',
         'user_key',
         'token_owner_key',
+        'send_welcome_message',
         )
 
     def __init__(self, mlist, subscriber=None, *,
-                 pre_verified=False, pre_confirmed=False, pre_approved=False):
+                 pre_verified=False, pre_confirmed=False, pre_approved=False,
+                 invitation=False, send_welcome_message=None):
         super().__init__(mlist, subscriber)
-        self.pre_verified = pre_verified
-        self.pre_confirmed = pre_confirmed
-        self.pre_approved = pre_approved
+        # An invitation is already supposed to be "pre_approved" and by
+        # confirming the invite a user will "confirm" and "verify" their
+        # subscription, hence these values are set to `True` if this is
+        # an invitation.
+        self.pre_verified = invitation or pre_verified
+        self.pre_confirmed = invitation or pre_confirmed
+        self.pre_approved = invitation or pre_approved
+        self.invitation = invitation
+        self.send_welcome_message = send_welcome_message
 
     def _step_sanity_checks(self):
         # Ensure that we have both an address and a user, even if the address
@@ -218,6 +228,9 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         # Is this email address banned?
         if IBanManager(self.mlist).is_banned(self.address.email):
             raise MembershipIsBannedError(self.mlist, self.address.email)
+        # Don't allow the list posting address.
+        if self.address.email.lower() == self.mlist.posting_address:
+            raise InvalidEmailAddressError('List posting address not allowed')
         # Check if there is already a subscription request for this email.
         pendings = getUtility(IPendings).find(
             mlist=self.mlist,
@@ -229,6 +242,10 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         self.push('verification_checks')
 
     def _step_verification_checks(self):
+        # If this is an invitation, send it now.
+        if self.invitation:
+            self.push('send_invitation')
+            return
         # Is the address already verified, or is the pre-verified flag set?
         if self.address.verified_on is None:
             if self.pre_verified:
@@ -323,7 +340,8 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
 
     def _step_do_subscription(self):
         # We can immediately subscribe the user to the mailing list.
-        self.member = self.mlist.subscribe(self.subscriber)
+        self.member = self.mlist.subscribe(
+            self.subscriber, send_welcome_message=self.send_welcome_message)
         assert self.token is None and self.token_owner is TokenOwner.no_one, (
             'Unexpected active token at end of subscription workflow')
 
@@ -337,6 +355,16 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
         # Now we wait for the confirmation.
         raise StopIteration
 
+    def _step_send_invitation(self):
+        self._set_token(TokenOwner.subscriber)
+        self.push('do_confirm_verify')
+        self.save()
+        # Triggering this event causes the confirmation message to be sent.
+        notify(SubscriptionInvitationNeededEvent(
+            self.mlist, self.token, self.address.email))
+        # Now we wait for the confirmation.
+        raise StopIteration
+
     def _step_do_confirm_verify(self):
         # Restore a little extra state that can't be stored in the database
         # (because the order of setattr() on restore is indeterminate), then
@@ -359,7 +387,7 @@ class SubscriptionWorkflow(_SubscriptionWorkflowCommon):
                      if self.mlist.subscription_policy in (
                          SubscriptionPolicy.moderate,
                          SubscriptionPolicy.confirm_then_moderate,
-                         )
+                         ) and not self.invitation
                      else 'do_subscription')
         self.push(next_step)
 
@@ -389,7 +417,9 @@ class UnSubscriptionWorkflow(_SubscriptionWorkflowCommon):
         self.pre_approved = pre_approved
 
     def _step_subscription_checks(self):
-        assert self.mlist.is_subscribed(self.subscriber)
+        # XXX assert self.member is not None is sufficient, but ...
+        assert (self.member is not None and
+                self.mlist.is_subscribed(self.member.subscriber))
         self.push('confirmation_checks')
 
     def _step_confirmation_checks(self):
@@ -510,13 +540,16 @@ class SubscriptionManager:
         self._mlist = mlist
 
     def register(self, subscriber=None, *,
-                 pre_verified=False, pre_confirmed=False, pre_approved=False):
+                 pre_verified=False, pre_confirmed=False, pre_approved=False,
+                 invitation=False, send_welcome_message=None):
         """See `ISubscriptionManager`."""
         workflow = SubscriptionWorkflow(
             self._mlist, subscriber,
             pre_verified=pre_verified,
             pre_confirmed=pre_confirmed,
-            pre_approved=pre_approved)
+            pre_approved=pre_approved,
+            invitation=invitation,
+            send_welcome_message=send_welcome_message)
         list(workflow)
         return workflow.token, workflow.token_owner, workflow.member
 
@@ -557,7 +590,19 @@ class SubscriptionManager:
 
 
 def _handle_confirmation_needed_events(event, template_name):
-    subject = 'confirm {}'.format(event.token)
+    # This function handles sending the confirmation email to the user
+    # for both subscriptions requiring confirmation and invitations
+    # requiring acceptance.
+    if template_name.endswith(':invite'):
+        subject = _('You have been invited to join the '
+                    '$event.mlist.fqdn_listname mailing list.')
+    elif template_name.endswith(':unsubscribe'):
+        subject = _('Your confirmation is needed to leave the '
+                    '$event.mlist.fqdn_listname mailing list.')
+    else:
+        assert(template_name.endswith(':subscribe'))
+        subject = _('Your confirmation is needed to join the '
+                    '$event.mlist.fqdn_listname mailing list.')
     confirm_address = event.mlist.confirm_address(event.token)
     email_address = event.email
     # Send a verification email to the address.
@@ -576,6 +621,8 @@ def _handle_confirmation_needed_events(event, template_name):
     msg = UserNotification(
         email_address, confirm_address, subject, text,
         event.mlist.preferred_language)
+    del msg['auto-submitted']
+    msg['Auto-Submitted'] = 'auto-generated'
     msg.send(event.mlist, add_precedence=False)
 
 
@@ -586,6 +633,13 @@ def handle_SubscriptionConfirmationNeededEvent(event):
     _handle_confirmation_needed_events(event, 'list:user:action:subscribe')
 
 
+@public
+def handle_SubscriptionInvitationNeededEvent(event):
+    if not isinstance(event, SubscriptionInvitationNeededEvent):
+        return
+    _handle_confirmation_needed_events(event, 'list:user:action:invite')
+
+
 @public
 def handle_UnsubscriptionConfirmationNeededEvent(event):
     if not isinstance(event, UnsubscriptionConfirmationNeededEvent):
