@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2007-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -42,6 +42,7 @@ class DiscardChain(TerminalChainBase):
         This writes a log message, fires a Zope event and then throws the
         message away.
         """
-        log.info('DISCARD: %s', msg.get('message-id', 'n/a'))
+        log.info('DISCARD: %s; %s', msg.get('message-id', 'n/a'),
+                 msgdata.get('moderation_reasons', '[n/a]'))
         notify(DiscardEvent(mlist, msg, msgdata, self))
         # Nothing more needs to happen.
