@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -200,6 +200,70 @@ A message body.
              ('baz', 'z+', LinkAction.jump, 'accept'),
             ])                                      # noqa: E124
 
+    def test_list_complex_rule_deletion(self):
+        # Test that the mailing-list header-match complex rules are read
+        # properly after deletion.
+        chain = config.chains['header-match']
+        header_matches = IHeaderMatchList(self._mlist)
+        header_matches.append('Foo', 'a+', 'reject')
+        header_matches.append('Bar', 'b+', 'discard')
+        header_matches.append('Baz', 'z+', 'accept')
+        links = [link for link in chain.get_links(self._mlist, Message(), {})
+                 if link.rule.name != 'any']
+        self.assertEqual(len(links), 3)
+        self.assertEqual([
+            (link.rule.header, link.rule.pattern, link.action, link.chain.name)
+            for link in links
+            ],
+            [('foo', 'a+', LinkAction.jump, 'reject'),
+             ('bar', 'b+', LinkAction.jump, 'discard'),
+             ('baz', 'z+', LinkAction.jump, 'accept'),
+            ])                                      # noqa: E124
+        del header_matches[0]
+        links = [link for link in chain.get_links(self._mlist, Message(), {})
+                 if link.rule.name != 'any']
+        self.assertEqual(len(links), 2)
+        self.assertEqual([
+            (link.rule.header, link.rule.pattern, link.action, link.chain.name)
+            for link in links
+            ],
+            [('bar', 'b+', LinkAction.jump, 'discard'),
+             ('baz', 'z+', LinkAction.jump, 'accept'),
+            ])                                      # noqa: E124
+
+    def test_list_complex_rule_reorder(self):
+        # Test that the mailing-list header-match complex rules are read
+        # properly after reordering.
+        chain = config.chains['header-match']
+        header_matches = IHeaderMatchList(self._mlist)
+        header_matches.append('Foo', 'a+', 'reject')
+        header_matches.append('Bar', 'b+', 'discard')
+        header_matches.append('Baz', 'z+', 'accept')
+        links = [link for link in chain.get_links(self._mlist, Message(), {})
+                 if link.rule.name != 'any']
+        self.assertEqual(len(links), 3)
+        self.assertEqual([
+            (link.rule.header, link.rule.pattern, link.action, link.chain.name)
+            for link in links
+            ],
+            [('foo', 'a+', LinkAction.jump, 'reject'),
+             ('bar', 'b+', LinkAction.jump, 'discard'),
+             ('baz', 'z+', LinkAction.jump, 'accept'),
+            ])                                      # noqa: E124
+        del header_matches[0]
+        header_matches.append('Foo', 'a+', 'reject')
+        links = [link for link in chain.get_links(self._mlist, Message(), {})
+                 if link.rule.name != 'any']
+        self.assertEqual(len(links), 3)
+        self.assertEqual([
+            (link.rule.header, link.rule.pattern, link.action, link.chain.name)
+            for link in links
+            ],
+            [('bar', 'b+', LinkAction.jump, 'discard'),
+             ('baz', 'z+', LinkAction.jump, 'accept'),
+             ('foo', 'a+', LinkAction.jump, 'reject'),
+            ])                                      # noqa: E124
+
     def test_header_in_subpart(self):
         # Test that headers in sub-parts are also matched.
         msg = mfs("""\
@@ -240,6 +304,30 @@ This is junk
         self.assertIsInstance(event, HoldEvent)
         self.assertEqual(event.chain, config.chains['hold'])
 
+    def test_rfc2047_encodedheader(self):
+        # Test case where msg.get_all() returns raw rfc2047 encoded string.
+        msg = message_from_bytes(b"""\
+From: anne@example.com
+To: test@example.com
+Subject: =?utf-8?b?SSBsaWtlIElrZQo=?=
+Message-ID: <ant>
+
+body
+
+""", Message)
+        msgdata = {}
+        header_matches = IHeaderMatchList(self._mlist)
+        header_matches.append('Subject', 'I Like Ike', 'hold')
+        # This event subscriber records the event that occurs when the message
+        # is processed by the owner chain.
+        events = []
+        with event_subscribers(events.append):
+            process(self._mlist, msg, msgdata, start_chain='header-match')
+        self.assertEqual(len(events), 1)
+        event = events[0]
+        self.assertIsInstance(event, HoldEvent)
+        self.assertEqual(event.chain, config.chains['hold'])
+
     def test_get_all_returns_non_string(self):
         # Test case where msg.get_all() returns header instance.
         msg = message_from_bytes(b"""\
@@ -347,12 +435,18 @@ A message body.
             self.assertEqual(event.mlist, self._mlist)
             self.assertEqual(event.msg, msg)
 
+    @unittest.expectedFailure
     @configuration('antispam', header_checks="""
     Header1: a+
     """, jump_chain='hold')
     def test_reuse_rules(self):
         # Test that existing header-match rules are used instead of creating
         # new ones.
+        # MAS Reusing existing rules is problematic. If the rule with say
+        # position = 0 is deleted the following rule should become
+        # header-match-<list-id>-0 but that rule has the old values for
+        # header and pattern.
+        # See https://gitlab.com/mailman/mailman/-/issues/818
         chain = config.chains['header-match']
         header_matches = IHeaderMatchList(self._mlist)
         header_matches.append('Header2', 'b+')
@@ -367,8 +461,8 @@ A message body.
         links_2 = get_links()
         # The link rules both have the same name...
         self.assertEqual(
-            [l.rule.name for l in links_1],
-            [l.rule.name for l in links_2],
+            [link.rule.name for link in links_1],
+            [link.rule.name for link in links_2],
             )
         # ...and are actually the identical objects.
         for link1, link2 in zip(links_1, links_2):
