@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -34,9 +34,14 @@ from zope.component import getUtility
 from zope.interface import implementer
 
 
-# A fake Bouncer class from Mailman 2.1, we don't use it but there are
-# instances in the .pck files.
-class Bouncer:
+# A fake module to go with `Bouncer`.
+class _Mailman:
+    __path__ = 'src/mailman/commands/cli_import.py'
+
+
+# A fake Mailman object with Bouncer class from Mailman 2.1, we don't use it
+# but there are instances in the .pck files.
+class _Bouncer:
     class _BounceInfo:
         pass
 
@@ -44,24 +49,32 @@ class Bouncer:
 @click.command(
     cls=I18nCommand,
     help=_("""\
-    Import Mailman 2.1 list data'.  Requires the fully-qualified name of the
+    Import Mailman 2.1 list data.  Requires the fully-qualified name of the
     list to import and the path to the Mailman 2.1 pickle file."""))
+@click.option(
+    '--charset', '-c', default='utf-8',
+    help=_("""\
+    Specify the encoding of strings in PICKLE_FILE if not utf-8 or a subset
+    thereof. This will normally be the Mailman 2.1 charset of the list's
+    preferred_language."""))
 @click.argument('listspec')
 @click.argument(
     'pickle_file', metavar='PICKLE_FILE',
     type=click.File(mode='rb'))
 @click.pass_context
-def import21(ctx, listspec, pickle_file):
+def import21(ctx, charset, listspec, pickle_file):
     mlist = getUtility(IListManager).get(listspec)
     if mlist is None:
         ctx.fail(_('No such list: $listspec'))
     with ExitStack() as resources:
-        resources.enter_context(hacked_sys_modules('Mailman.Bouncer', Bouncer))
+        resources.enter_context(hacked_sys_modules('Mailman', _Mailman))
+        resources.enter_context(
+            hacked_sys_modules('Mailman.Bouncer', _Bouncer))
         resources.enter_context(transaction())
         while True:
             try:
                 config_dict = pickle.load(
-                    pickle_file, encoding='utf-8', errors='ignore')
+                    pickle_file, encoding=charset, errors='ignore')
             except EOFError:
                 break
             except pickle.UnpicklingError:
