@@ -1,4 +1,4 @@
-# Copyright (C) 2002-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2002-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -17,12 +17,15 @@
 
 """The email commands 'join' and 'subscribe'."""
 
+from email.header import decode_header, make_header
 from email.utils import formataddr, parseaddr
 from mailman.core.i18n import _
+from mailman.interfaces.address import InvalidEmailAddressError
 from mailman.interfaces.command import ContinueProcessing, IEmailCommand
-from mailman.interfaces.member import DeliveryMode, MemberRole
+from mailman.interfaces.member import (
+    AlreadySubscribedError, DeliveryMode, MembershipIsBannedError)
 from mailman.interfaces.subscriptions import (
-    ISubscriptionManager, ISubscriptionService)
+    ISubscriptionManager, SubscriptionPendingError)
 from mailman.interfaces.usermanager import IUserManager
 from public import public
 from zope.component import getUtility
@@ -57,24 +60,34 @@ class Join:
 
     name = 'join'
     # XXX 2012-02-29 BAW: DeliveryMode.summary is not yet supported.
-    argument_description = '[digest=<no|mime|plain>]'
+    argument_description = ('[digest=<no|mime|plain>] '
+                            '[address=user@example.com]')
     description = _("""\
 You will be asked to confirm your subscription request and you may be issued a
 provisional password.
 
 By using the 'digest' option, you can specify whether you want digest delivery
 or not.  If not specified, the mailing list's default delivery mode will be
-used.
+used.  You can use the 'address' option to request subscription of an address
+other than the sender of the command.
 """)
     short_description = _('Join this mailing list.')
 
     def process(self, mlist, msg, msgdata, arguments, results):
         """See `IEmailCommand`."""
         # Parse the arguments.
-        delivery_mode = self._parse_arguments(arguments, results)
+        delivery_mode, address = self._parse_arguments(arguments, results)
         if delivery_mode is ContinueProcessing.no:
             return ContinueProcessing.no
-        display_name, email = parseaddr(msg['from'])
+        if not address:
+            # RFC 2047 decode the From: header.
+            if msg['from'] is None:
+                display_name, email = ('', '')
+            else:
+                decoded_from = str(make_header(decode_header(msg['from'])))
+                display_name, email = parseaddr(decoded_from)
+        else:
+            display_name, email = ('', address)
         # Address could be None or the empty string.
         if not email:
             email = msg.sender
@@ -93,16 +106,23 @@ used.
         joins.add(email)
         results.joins = joins
         person = formataddr((display_name, email))                # noqa: F841
-        # Is this person already a member of the list?  Search for all
-        # matching memberships.
-        members = getUtility(ISubscriptionService).find_members(
-            email, mlist.list_id, MemberRole.member)
-        if len(members) > 0:
-            print(_('$person is already a member'), file=results)
+        try:
+            subscriber = match_subscriber(email, display_name)
+        except InvalidEmailAddressError as e:
+            print('Invalid email address: {}'.format(e), file=results)
             return ContinueProcessing.yes
-        subscriber = match_subscriber(email, display_name)
-        ISubscriptionManager(mlist).register(subscriber)
-        print(_('Confirmation email sent to $person'), file=results)
+        try:
+            ISubscriptionManager(mlist).register(subscriber)
+        except (AlreadySubscribedError, InvalidEmailAddressError,
+                MembershipIsBannedError) as e:
+            print(str(e), file=results)
+        except SubscriptionPendingError:
+            # SubscriptionPendingError doesn't return an error message.
+            listname = mlist.fqdn_listname                        # noqa: F841
+            print(_('$person has a pending subscription for $listname'),
+                  file=results)
+        else:
+            print(_('Confirmation email sent to $person'), file=results)
         return ContinueProcessing.yes
 
     def _parse_arguments(self, arguments, results):
@@ -111,25 +131,30 @@ used.
         :param arguments: The sequences of arguments as given to the
             `process()` method.
         :param results: The results object.
-        :return: The delivery mode, None, or ContinueProcessing.no on error.
+        :return: A tuple (delivery mode, address) or
+            (ContinueProcessing.no, None) on error.
         """
         mode = DeliveryMode.regular
+        address = None
         for argument in arguments:
             parts = argument.split('=', 1)
-            if len(parts) != 2 or parts[0] != 'digest':
+            if len(parts) != 2 or parts[0] not in ('digest', 'address'):
                 print(self.name, _('bad argument: $argument'),
                       file=results)
-                return ContinueProcessing.no
-            mode = {
-                'no': DeliveryMode.regular,
-                'plain': DeliveryMode.plaintext_digests,
-                'mime': DeliveryMode.mime_digests,
-                }.get(parts[1])
-            if mode is None:
-                print(self.name, _('bad argument: $argument'),
-                      file=results)
-                return ContinueProcessing.no
-        return mode
+                return (ContinueProcessing.no, None)
+            if parts[0] == 'digest':
+                mode = {
+                    'no': DeliveryMode.regular,
+                    'plain': DeliveryMode.plaintext_digests,
+                    'mime': DeliveryMode.mime_digests,
+                    }.get(parts[1])
+                if mode is None:
+                    print(self.name, _('bad argument: $argument'),
+                          file=results)
+                    return (ContinueProcessing.no, None)
+            if parts[0] == 'address':
+                address = parts[1]
+        return (mode, address)
 
 
 @public
