@@ -1,4 +1,4 @@
-# Copyright (C) 2013-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2013-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -21,6 +21,7 @@ import unittest
 
 from click.testing import CliRunner
 from mailman.commands.cli_conf import conf
+from mailman.config import config
 from mailman.testing.layers import ConfigLayer
 from tempfile import NamedTemporaryFile
 
@@ -39,7 +40,7 @@ class TestConf(unittest.TestCase):
         self.assertEqual(
             result.output,
             'Usage: conf [OPTIONS]\n'
-            'Try "conf --help" for help.\n\n'
+            'Try \'conf --help\' for help.\n\n'
             'Error: No such section: thissectiondoesnotexist\n')
 
     def test_cannot_access_nonexistent_section_and_key(self):
@@ -49,7 +50,7 @@ class TestConf(unittest.TestCase):
         self.assertEqual(
             result.output,
             'Usage: conf [OPTIONS]\n'
-            'Try "conf --help" for help.\n\n'
+            'Try \'conf --help\' for help.\n\n'
             'Error: No such section: thissectiondoesnotexist\n')
 
     def test_cannot_access_nonexistent_key(self):
@@ -59,9 +60,18 @@ class TestConf(unittest.TestCase):
         self.assertEqual(
             result.output,
             'Usage: conf [OPTIONS]\n'
-            'Try "conf --help" for help.\n\n'
+            'Try \'conf --help\' for help.\n\n'
             'Error: Section mailman: No such key: thiskeydoesnotexist\n')
 
+    def test_pushed_section_is_found(self):
+        config.push('test config', """\
+[archiver.other]
+enable: yes
+""")
+        result = self._command.invoke(conf, ('-k', 'enable'))
+        self.assertIn('[archiver.other] enable: yes', result.output)
+        config.pop('test config')
+
     def test_output_to_explicit_stdout(self):
         result = self._command.invoke(
             conf, ('-o', '-', '-s', 'shell', '-k', 'use_ipython'))
