@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -22,6 +22,7 @@ import unittest
 from email import message_from_binary_file
 from email.header import Header
 from email.parser import FeedParser
+from email.utils import _has_surrogates
 from importlib_resources import path
 from mailman.app.lifecycle import create_list
 from mailman.email.message import Message, UserNotification
@@ -53,6 +54,12 @@ class TestMessage(unittest.TestCase):
         self.assertEqual(items[0].msg.get_all('precedence'),
                          ['omg wtf bbq'])
 
+    def test_reduced_rfc_2369_headers(self):
+        # Notifications should get reduced List-* headers.
+        self._msg.send(self._mlist)
+        items = get_queue_messages('virgin', expected_count=1)
+        self.assertTrue(items[0].msgdata.get('reduced_list_headers'))
+
 
 class TestMessageSubclass(unittest.TestCase):
     layer = ConfigLayer
@@ -115,6 +122,12 @@ Test content
                 text = fp.read().decode('ascii', 'replace')
         self.assertEqual(msg.as_string(), text)
 
+    def test_as_string_unicode_surrogates(self):
+        with path('mailman.email.tests.data', 'bad_email_4.eml') as email_path:
+            with open(str(email_path), 'rb') as fp:
+                msg = message_from_binary_file(fp, Message)
+        self.assertFalse(_has_surrogates(msg.as_string()))
+
     def test_bogus_content_charset(self):
         with path('mailman.email.tests.data', 'bad_email_3.eml') as email_path:
             with open(str(email_path), 'rb') as fp:
