@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2007-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -90,6 +90,15 @@ class SubscriptionPolicy(Enum):
     confirm_then_moderate = 3
 
 
+@public
+class ArchiveRenderingMode(Enum):
+    """Email rendering mode in Archiver."""
+    # Default text.
+    text = 1
+    # Render emails as markdown text.
+    markdown = 2
+
+
 @public
 class IMailingList(Interface):
     """A mailing list."""
@@ -367,6 +376,8 @@ class IMailingList(Interface):
             that it is possible to subscribe an address to a mailing list with
             a particular role, and also subscribe a user with a matching
             preferred address that is explicitly subscribed with the same role.
+        :raises InvalidEmailAddressError: If the address being subscribed is
+            the list's posting address.
         """
 
     # Delivery.
@@ -647,10 +658,44 @@ class IMailingList(Interface):
         without any other checks.
         """)
 
-    newsgroup_moderation = Attribute(
-        """The moderation policy for the linked newsgroup, if there is one.""")
-
     # Bounces.
+    bounce_info_stale_after = Attribute(
+        """Number of days after which bounce info is considered stale.
+
+        The number of days after which a member's bounce information is
+        considered stale. If no new bounces have been received in the interim,
+        the bounce score is reset to zero. This value must be an integer.
+        """)
+
+    bounce_notify_owner_on_disable = Attribute(
+        """This option controls whether or not the list owner
+        is notified when a member's subscription is automatically
+        disabled due to their bounce threshold being reached.
+        """)
+
+    bounce_notify_owner_on_removal = Attribute(
+        """This option controls whether or not the list owner
+        is notified when a member is removed from the list after
+        their disabled notifications have been exhausted.
+        """)
+
+    bounce_score_threshold = Attribute(
+        """Bounce score threshold for a mailing-list.
+
+        Threshold value for a Member's bounce_score after which we will either
+        disable membership or unsubscribe the member from the mailing list.
+        """)
+
+    bounce_you_are_disabled_warnings = Attribute(
+        """The number of notices a disabled member will
+        receive before their address is removed from the
+        mailing list's roster. Set this to 0 to immediately
+        remove an address from the list once their bounce
+        score exceeds the threshold. This value must be an integer.
+        """)
+
+    bounce_you_are_disabled_warnings_interval = Attribute(
+        """The number of days between each disabled notification.""")
 
     forward_unrecognized_bounces_to = Attribute(
         """What to do when a bounce contains no recognizable addresses.
@@ -687,6 +732,42 @@ class IMailingList(Interface):
     send_goodbye_message = Attribute(
         """Flag indicating whether a goodbye message should be sent.""")
 
+    # Usenet gateway.
+
+    gateway_to_mail = Attribute(
+        """Flag indicating that posts to the linked newsgroup should be gated
+        to the list.""")
+
+    gateway_to_news = Attribute(
+        """Flag indicating that posts to the list should be gated to the
+        linked newsgroup.""")
+
+    linked_newsgroup = Attribute(
+        """The name of the linked newsgroup.""")
+
+    newsgroup_moderation = Attribute(
+        """The moderation policy for the linked newsgroup, if there is one.""")
+
+    nntp_prefix_subject_too = Attribute(
+        """Flag indicating whether the list's subject_prefix should be included
+        in posts gated to usenet.""")
+
+    usenet_watermark = Attribute(
+        """The NNTP server's message number of the last post gated to the
+        list.""")
+
+    # Hyperkitty specific settings.
+
+    archive_rendering_mode = Attribute(
+        """How are Emails rendered in archivers?  This is only a suggested mode
+        for Archivers and is not related to actual contents of Emails.
+
+        Primary use of this setting is going to be Hyperkitty, which can render
+        emails as markdown text.
+
+        Values are `ArchiveRenderingMode` Enum.
+        """)
+
 
 @public
 class IAcceptableAlias(Interface):
@@ -789,6 +870,14 @@ class IHeaderMatch(Interface):
         file is used.
         """)
 
+    tag = Attribute(
+        """An arbitrary value to identify a set of IHeaderMatches.
+
+        This tag can be used to filter a set of IHeaderMatches, so that they
+        can set and removed together, without having to query the whole list
+        and iterate over them.
+        """)
+
 
 @public
 class IHeaderMatchList(Interface):
@@ -797,7 +886,7 @@ class IHeaderMatchList(Interface):
     def clear():
         """Clear the list of header matching rules."""
 
-    def append(header, pattern, chain=None):
+    def append(header, pattern, chain=None, tag=None):
         """Append the given rule to this mailing list's header match list.
 
         :param header: The email header to filter on.  It will be converted to
@@ -808,11 +897,13 @@ class IHeaderMatchList(Interface):
         :param chain: The chain to jump to, or None to use the site-wide
             antispam jump chain via the configuration.  Defaults to None.
         :type chain: string or None
+        :param tag: An arbitrary value to identify a set of IHeaderMatches.
+        :type tag: string or None
         :raises ValueError: if the header/pattern pair already exists for this
             mailing list.
         """
 
-    def insert(index, header, pattern, chain=None):
+    def insert(index, header, pattern, chain=None, tag=None):
         """Insert a header match rule.
 
         Inserts the given rule at the given index position in this
@@ -828,6 +919,8 @@ class IHeaderMatchList(Interface):
         :param chain: The chain to jump to, or None to use the site-wide
             antispam jump chain via the configuration.  Defaults to None.
         :type chain: string or None
+        :param tag: An arbitrary value to identify a set of IHeaderMatches.
+        :type tag: string or None
         :raises ValueError: if the header/pattern pair already exists for this
             mailing list.
         """
