@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2007-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -21,13 +21,13 @@ The request database handles events that must be approved by the list
 moderators, such as subscription requests and held messages.
 """
 
-from enum import Enum
+from enum import IntEnum
 from public import public
 from zope.interface import Attribute, Interface
 
 
 @public
-class RequestType(Enum):
+class RequestType(IntEnum):
     held_message = 1
     subscription = 2
     unsubscription = 3
