@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2008-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -43,7 +43,7 @@ class Domain(Model):
 
     id = Column(Integer, primary_key=True)
 
-    mail_host = Column(SAUnicode)
+    mail_host = Column(SAUnicode, unique=True)
     description = Column(SAUnicode)
     owners = relationship('User',
                           secondary='domain_owner',
