@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -234,12 +234,12 @@ Message-ID: <argon>
             type='subscription',
             list_id='bee.example.com')
         pendingdb.add(pendable_2)
-        self.assertEqual(pendingdb.count, 2)
+        self.assertEqual(pendingdb.count(), 2)
         list_manager = getUtility(IListManager)
         list_manager.delete(self._ant)
-        self.assertEqual(pendingdb.count, 1)
+        self.assertEqual(pendingdb.count(), 1)
         list_manager.delete(self._bee)
-        self.assertEqual(pendingdb.count, 0)
+        self.assertEqual(pendingdb.count(), 0)
 
 
 class TestListCreation(unittest.TestCase):
