@@ -1,4 +1,4 @@
-# Copyright (C) 2007-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2007-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -159,4 +159,5 @@ class UserManager:
     @dbconnection
     def server_owners(self, store):
         """ See `IUserManager."""
-        yield from store.query(User).filter_by(is_server_owner=True)
+        yield from store.query(User).filter_by(
+            is_server_owner=True).order_by(User.id)
