@@ -1,4 +1,4 @@
-# Copyright (C) 2009-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2009-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -22,9 +22,10 @@ import socket
 import logging
 import smtplib
 
+from lazr.config import as_boolean
 from mailman.config import config
 from mailman.interfaces.mta import IMailTransportAgentDelivery
-from mailman.mta.connection import Connection
+from mailman.mta.connection import Connection, as_SecureMode
 from public import public
 from zope.interface import implementer
 
@@ -39,12 +40,15 @@ class BaseDelivery:
 
     def __init__(self):
         """Create a basic deliverer."""
-        username = (config.mta.smtp_user if config.mta.smtp_user else None)
-        password = (config.mta.smtp_pass if config.mta.smtp_pass else None)
         self._connection = Connection(
             config.mta.smtp_host, int(config.mta.smtp_port),
             int(config.mta.max_sessions_per_connection),
-            username, password)
+            config.mta.smtp_user if config.mta.smtp_user else None,
+            config.mta.smtp_pass if config.mta.smtp_pass else None,
+            as_SecureMode(config.mta.smtp_secure_mode),
+            as_boolean(config.mta.smtp_verify_cert),
+            as_boolean(config.mta.smtp_verify_hostname),
+            )
 
     def _deliver_to_recipients(self, mlist, msg, msgdata, recipients):
         """Low-level delivery to a set of recipients.
