@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -119,7 +119,10 @@ class HeldMessage(_HeldMessageBase):
 
     def on_post(self, request, response):
         try:
-            validator = Validator(action=enum_validator(Action))
+            validator = Validator(
+                action=enum_validator(Action),
+                comment=str,
+                _optional=('comment',))
             arguments = validator(request)
         except ValueError as error:
             bad_request(response, str(error))
@@ -138,6 +141,17 @@ class HeldMessage(_HeldMessageBase):
             no_content(response)
 
 
+class _HeldMessageCount:
+
+    def __init__(self, mlist):
+        self._mlist = mlist
+
+    def on_get(self, request, response):
+        requests = IListRequests(self._mlist)
+        count = requests.count_of(RequestType.held_message)
+        okay(response, etag(dict(count=count)))
+
+
 @public
 class HeldMessages(_HeldMessageBase, CollectionMixin):
     """Resource for messages held for moderation."""
@@ -160,6 +174,11 @@ class HeldMessages(_HeldMessageBase, CollectionMixin):
         resource = self._make_collection(request)
         okay(response, etag(resource))
 
+    @child()
+    def count(self, context, segments):
+        """/lists/listname/held/count"""
+        return _HeldMessageCount(self._mlist)
+
     @child(r'^(?P<id>[^/]+)')
     def message(self, context, segments, **kw):
         return HeldMessage(self._mlist, kw['id'])
