@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -21,8 +21,8 @@ from mailman.app.moderator import send_rejection
 from mailman.core.i18n import _
 from mailman.interfaces.action import Action
 from mailman.interfaces.member import AlreadySubscribedError
-from mailman.interfaces.pending import IPendings
-from mailman.interfaces.subscriptions import ISubscriptionManager
+from mailman.interfaces.pending import IPendings, PendType
+from mailman.interfaces.subscriptions import ISubscriptionManager, TokenOwner
 from mailman.rest.helpers import (
     CollectionMixin, bad_request, child, conflict, etag, no_content,
     not_found, okay)
@@ -70,7 +70,9 @@ class IndividualRequest(_ModerationBase):
 
     def on_post(self, request, response):
         try:
-            validator = Validator(action=enum_validator(Action))
+            validator = Validator(action=enum_validator(Action),
+                                  reason=str,
+                                  _optional=('reason',))
             arguments = validator(request)
         except ValueError as error:
             bad_request(response, str(error))
@@ -107,10 +109,47 @@ class IndividualRequest(_ModerationBase):
                 not_found(response)
             else:
                 no_content(response)
+                reason = arguments.get('reason', _('[No reason given]'))
                 send_rejection(
                     self._mlist, _('Subscription request'),
-                    pendable['email'],
-                    _('[No reason given]'))
+                    pendable['email'], reason)
+
+
+class _SubscriptionRequestsFound(_ModerationBase, CollectionMixin):
+    """An abstract class to return a sub-set of subscription requests.
+
+    This is used for filtering subscription requests based on token_owner.
+    """
+    def __init__(self, sub_requests):
+        super().__init__()
+        self._requests = sub_requests
+
+    def _get_collection(self, requests):
+        return self._requests
+
+
+class _SubscriptionRequestCount:
+
+    def __init__(self, mlist):
+        self._mlist = mlist
+
+    def on_get(self, request, response):
+        validator = Validator(
+            token_owner=enum_validator(TokenOwner),
+            request_type=enum_validator(PendType),
+            _optional=['token_owner', 'request_type'])
+        try:
+            data = validator(request)
+        except ValueError as error:
+            bad_request(response, str(error))
+        else:
+            token_owner = data.pop('token_owner', None)
+            pend_type = data.pop('request_type', PendType.subscription)
+            count = getUtility(IPendings).count(
+                mlist=self._mlist,
+                pend_type=pend_type.name,
+                token_owner=token_owner)
+            okay(response, etag(dict(count=count)))
 
 
 @public
@@ -121,15 +160,36 @@ class SubscriptionRequests(_ModerationBase, CollectionMixin):
         super().__init__()
         self._mlist = mlist
 
-    def _get_collection(self, request):
-        pendings = getUtility(IPendings).find(
-            mlist=self._mlist, pend_type='subscription')
-        return [token for token, pendable in pendings]
-
     def on_get(self, request, response):
         """/lists/listname/requests"""
-        resource = self._make_collection(request)
-        okay(response, etag(resource))
+        validator = Validator(
+            token_owner=enum_validator(TokenOwner),
+            request_type=enum_validator(PendType),
+            page=int,
+            count=int,
+            _optional=['token_owner', 'page', 'count', 'request_type'],
+            )
+
+        try:
+            data = validator(request)
+        except ValueError as error:
+            bad_request(response, str(error))
+        else:
+            data.pop('page', None)
+            data.pop('count', None)
+            token_owner = data.pop('token_owner', None)
+            pend_type = data.pop('request_type', PendType.subscription)
+            pendings = getUtility(IPendings).find(
+                mlist=self._mlist,
+                pend_type=pend_type.name,
+                token_owner=token_owner)
+            resource = _SubscriptionRequestsFound(
+                [token for token, pendable in pendings])
+            okay(response, etag(resource._make_collection(request)))
+
+    @child()
+    def count(self, context, segments):
+        return _SubscriptionRequestCount(self._mlist)
 
     @child(r'^(?P<token>[^/]+)')
     def subscription(self, context, segments, **kw):
