@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -38,6 +38,9 @@ RESOURCE = dict(
         'bee@example.com',
         'cat@example.com',
         ],
+    accept_these_nonmembers=[
+        r'ant_*@example.com',
+        ],
     admin_immed_notify=False,
     admin_notify_mchanges=True,
     administrivia=False,
@@ -45,6 +48,7 @@ RESOURCE = dict(
     allow_list_posts=False,
     anonymous_list=True,
     archive_policy='never',
+    archive_rendering_mode='text',
     autorespond_owner='respond_and_discard',
     autorespond_postings='respond_and_continue',
     autorespond_requests='respond_and_discard',
@@ -52,6 +56,12 @@ RESOURCE = dict(
     autoresponse_owner_text='the owner',
     autoresponse_postings_text='the mailing list',
     autoresponse_request_text='the robot',
+    bounce_info_stale_after='7d',
+    bounce_notify_owner_on_disable=False,
+    bounce_notify_owner_on_removal=True,
+    bounce_score_threshold=5,
+    bounce_you_are_disabled_warnings=3,
+    bounce_you_are_disabled_warnings_interval='2d',
     collapse_alternatives=False,
     convert_html_to_plaintext=True,
     default_member_action='hold',
@@ -61,30 +71,56 @@ RESOURCE = dict(
     digest_size_threshold=10.5,
     digest_volume_frequency='monthly',
     digests_enabled=True,
+    discard_these_nonmembers=[
+       'aperson@example.com',
+       ],
     display_name='Fnords',
     dmarc_mitigate_action='munge_from',
     dmarc_mitigate_unconditionally=False,
     dmarc_moderation_notice='Some moderation notice',
     dmarc_wrapped_message_text='some message text',
+    emergency=False,
+    filter_action='discard',
+    filter_extensions=['.exe'],
     filter_content=True,
+    filter_types=['application/zip'],
     first_strip_reply_to=True,
+    forward_unrecognized_bounces_to='administrators',
+    gateway_to_mail=False,
+    gateway_to_news=False,
     goodbye_message_uri='mailman:///goodbye.txt',
+    hold_these_nonmembers=[
+        r'*@example.com',
+        ],
     include_rfc2369_headers=False,
     info='This is the mailing list info',
+    linked_newsgroup='',
     moderator_password='password',
     max_message_size='150',
+    newsgroup_moderation='none',
+    nntp_prefix_subject_too=False,
+    pass_extensions=['.pdf'],
+    pass_types=['image/jpeg'],
+    personalize='none',
     posting_pipeline='virgin',
     preferred_language='en',
+    process_bounces=True,
+    reject_these_nonmembers=[
+       'bperson@example.com',
+       ],
     reply_goes_to_list='point_to_list',
     reply_to_address='bee@example.com',
     require_explicit_destination=True,
     member_roster_visibility='public',
+    send_goodbye_message=False,
     send_welcome_message=False,
     subject_prefix='[ant]',
     subscription_policy='confirm_then_moderate',
+    unsubscription_policy='confirm',
     welcome_message_uri='mailman:///welcome.txt',
     respond_to_post_requests=True,
     max_num_recipients=150,
+    max_days_to_hold=20,
     )
 
 
@@ -483,6 +519,28 @@ class TestConfiguration(unittest.TestCase):
                 'PATCH')
             self.assertEqual(self._mlist.info, '')
 
+    def test_patch_send_welcome_message(self):
+        with transaction():
+            self._mlist.send_welcome_message = False
+        resource, response = call_api(
+            'http://localhost:9001/3.0/lists/ant.example.com/config'
+            '/send_welcome_message',
+            dict(send_welcome_message=True),
+            'PATCH')
+        self.assertEqual(response.status_code, 204)
+        self.assertTrue(self._mlist.send_welcome_message)
+
+    def test_patch_send_goodbye_message(self):
+        with transaction():
+            self._mlist.send_goodbye_message = False
+        resource, response = call_api(
+            'http://localhost:9001/3.0/lists/ant.example.com/config'
+            '/send_goodbye_message',
+            dict(send_goodbye_message=True),
+            'PATCH')
+        self.assertEqual(response.status_code, 204)
+        self.assertTrue(self._mlist.send_goodbye_message)
+
     def test_delete_top_level_listconf(self):
         with self.assertRaises(HTTPError) as cm:
             call_api('http://localhost:9001/3.0/lists/ant.example.com/config',
