@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -29,8 +29,8 @@ from mailman.interfaces.subscriptions import ISubscriptionManager, TokenOwner
 from mailman.interfaces.usermanager import IUserManager
 from mailman.runners.incoming import IncomingRunner
 from mailman.testing.helpers import (
-    TestableMaster, call_api, get_lmtp_client, make_testable_runner,
-    set_preferred, subscribe, wait_for_webservice)
+    TestableMaster, call_api, get_lmtp_client, get_queue_messages,
+    make_testable_runner, set_preferred, subscribe, wait_for_webservice)
 from mailman.testing.layers import ConfigLayer, RESTLayer
 from mailman.utilities.datetime import now
 from urllib.error import HTTPError
@@ -71,7 +71,8 @@ class TestMembership(unittest.TestCase):
 
     def test_try_to_leave_a_list_twice(self):
         with transaction():
-            anne = self._usermanager.create_address('anne@example.com')
+            anne = self._usermanager.create_user('anne@example.com')
+            set_preferred(anne)
             self._mlist.subscribe(anne)
         url = 'http://localhost:9001/3.0/members/1'
         json, response = call_api(url, method='DELETE')
@@ -83,6 +84,76 @@ class TestMembership(unittest.TestCase):
             call_api(url, method='DELETE')
         self.assertEqual(cm.exception.code, 404)
 
+    def test_leave_mlist_sends_notice_to_user(self):
+        with transaction():
+            anne = self._usermanager.create_user('anne@example.com')
+            set_preferred(anne)
+            self._mlist.subscribe(anne)
+        # Empty the virgin queue.
+        get_queue_messages('virgin', expected_count=1)
+        url = 'http://localhost:9001/3.0/members/1'
+        # Calling the DELETE api should send user a notice.
+        json, response = call_api(url, method='DELETE')
+        self.assertEqual(json, None)
+        self.assertEqual(response.status_code, 204)
+        items = get_queue_messages('virgin', expected_count=1)
+        self.assertEqual(str(items[0].msg['to']), 'anne@example.com')
+        self.assertEqual(
+            str(items[0].msg['subject']),
+            'You have been unsubscribed from the Test mailing list')
+
+    def test_leave_mlist_moderate_policy(self):
+        with transaction():
+            self._mlist.unsubscription_policy = SubscriptionPolicy.moderate
+            anne = self._usermanager.create_user('anne@example.com')
+            set_preferred(anne)
+            self._mlist.subscribe(anne)
+        # Empty the virgin queue.
+        get_queue_messages('virgin', expected_count=1)
+        url = 'http://localhost:9001/3.0/members/1'
+        # Try unsubscribing the user.
+        with transaction():
+            json, resp = call_api(
+                url,
+                data=dict(pre_confirmed=True),
+                method='DELETE')
+        self.assertEqual(resp.status_code, 202)
+        self.assertEqual(json.get('token_owner'), 'moderator')
+        token = json.get('token')
+        # The member should not have been deleted.
+        json, resp = call_api(url)
+        self.assertEqual(resp.status_code, 200)
+        # Now, approve the un-subscription request.
+        json, resp = call_api(
+            'http://localhost:9001/3.0/lists/test.example.com/'
+            'requests/{}'.format(token),
+            data={'action': 'accept'})
+        self.assertEqual(resp.status_code, 204)
+        self.assertEqual(json, None)
+        # Finally, the user should be unsubscribed.
+        with self.assertRaises(HTTPError) as cm:
+            resp = call_api(url)
+        self.assertEqual(cm.exception.code, 404)
+
+    def test_leave_mlist_confirm_policy(self):
+        with transaction():
+            anne = self._usermanager.create_user('anne@example.com')
+            set_preferred(anne)
+            self._mlist.subscribe(anne)
+        # Empty the virgin queue.
+        get_queue_messages('virgin', expected_count=1)
+        url = 'http://localhost:9001/3.0/members/1'
+        # The default policy is to confirm, so an un-confirmed request would be
+        # held for user confirmation.
+        json, response = call_api(url, data={'pre_confirmed': False},
+                                  method='DELETE')
+        items = get_queue_messages('virgin', expected_count=1)
+        self.assertEqual(str(items[0].msg['to']), 'anne@example.com')
+        self.assertEqual(
+            str(items[0].msg['subject']),
+            'Your confirmation is needed to leave the test@example.com'
+            ' mailing list.')
+
     def test_try_to_join_a_list_twice(self):
         with transaction():
             anne = self._usermanager.create_address('anne@example.com')
@@ -113,6 +184,37 @@ class TestMembership(unittest.TestCase):
         self.assertEqual(cm.exception.code, 409)
         self.assertEqual(cm.exception.reason, 'Member already subscribed')
 
+    def test_invite_new_member(self):
+        # Invite an email address to join a list.
+        json, response = call_api('http://localhost:9001/3.0/members', {
+            'list_id': 'test.example.com',
+            'subscriber': 'anne@example.com',
+            'display_name': 'Anne Person',
+            'invitation': True,
+            })
+        self.assertEqual(response.status_code, 202)
+        self.assertEqual(len(json), 3)
+
+        # Response will always add http_etag.
+        fields = ['http_etag', 'token', 'token_owner']
+        self.assertEqual(sorted(json.keys()), fields)
+        self.assertEqual(json['token_owner'], 'subscriber')
+
+    def test_invite_existing_member(self):
+        # Invite an existing member's email address to join a list.
+        with transaction():
+            anne = self._usermanager.create_address('anne@example.com')
+            self._mlist.subscribe(anne)
+        with self.assertRaises(HTTPError) as cm:
+            call_api('http://localhost:9001/3.0/members', {
+                'list_id': 'test.example.com',
+                'subscriber': 'anne@example.com',
+                'display_name': 'Anne Person',
+                'invitation': True,
+                })
+        self.assertEqual(cm.exception.code, 409)
+        self.assertEqual(cm.exception.reason, 'Member already subscribed')
+
     def test_subscribe_user_without_preferred_address(self):
         with transaction():
             getUtility(IUserManager).create_user('anne@example.com')
@@ -437,6 +539,92 @@ class TestMembership(unittest.TestCase):
         self.assertEqual(cm.exception.code, 400)
         self.assertEqual(cm.exception.reason, 'Membership is banned')
 
+    def test_list_posting_address_as_member(self):
+        # The list posting address tries to subscribe.
+        with self.assertRaises(HTTPError) as cm:
+            call_api('http://localhost:9001/3.0/members', {
+                'list_id': 'test.example.com',
+                'subscriber': self._mlist.posting_address,
+                })
+        self.assertEqual(cm.exception.code, 400)
+        self.assertEqual(cm.exception.reason,
+                         'List posting address cannot subscribe')
+
+    def test_list_posting_address_as_member_approved(self):
+        # The list posting address tries to subscribe.
+        with self.assertRaises(HTTPError) as cm:
+            call_api('http://localhost:9001/3.0/members', {
+                'list_id': 'test.example.com',
+                'subscriber': self._mlist.posting_address,
+                'pre_verified': True,
+                'pre_confirmed': True,
+                'pre_approved': True,
+                })
+        self.assertEqual(cm.exception.code, 400)
+        self.assertEqual(cm.exception.reason,
+                         'List posting address cannot subscribe')
+
+    def test_list_posting_address_as_owner(self):
+        # The liust posting address tries to subscribe.
+        with self.assertRaises(HTTPError) as cm:
+            call_api('http://localhost:9001/3.0/members', {
+                'list_id': 'test.example.com',
+                'subscriber': self._mlist.posting_address,
+                'role': 'owner',
+                })
+        self.assertEqual(cm.exception.code, 400)
+        self.assertEqual(cm.exception.reason,
+                         'List posting address cannot be added')
+
+    def test_list_posting_address_as_moderator(self):
+        # The liust posting address tries to subscribe.
+        with self.assertRaises(HTTPError) as cm:
+            call_api('http://localhost:9001/3.0/members', {
+                'list_id': 'test.example.com',
+                'subscriber': self._mlist.posting_address,
+                'role': 'moderator'
+                })
+        self.assertEqual(cm.exception.code, 400)
+        self.assertEqual(cm.exception.reason,
+                         'List posting address cannot be added')
+
+    def test_subscribe_supress_welcome_message(self):
+        # Test subscription of a new member is able to supress welcome message,
+        # even if Mailinglist is configured to do so.
+        self._mlist.send_welcome_message = True
+        content, response = call_api('http://localhost:9001/3.0/members', {
+            'list_id': 'test.example.com',
+            'subscriber': 'ANNE@example.com',
+            'pre_verified': True,
+            'pre_confirmed': True,
+            'pre_approved': True,
+            'send_welcome_message': False,
+            })
+        self.assertEqual(response.status_code, 201)
+        member = self._mlist.members.get_member('anne@example.com')
+        self.assertIsNotNone(member)
+        get_queue_messages('virgin', expected_count=0)
+
+    def test_send_welcome_message(self):
+        # Test that the send_welcome_message flag sends welcome message, even
+        # when the mlist isn't configured to do so.
+        self._mlist.send_welcome_message = False
+        content, response = call_api('http://localhost:9001/3.0/members', {
+            'list_id': 'test.example.com',
+            'subscriber': 'ANNE@example.com',
+            'pre_verified': True,
+            'pre_confirmed': True,
+            'pre_approved': True,
+            'send_welcome_message': True,
+            })
+        self.assertEqual(response.status_code, 201)
+        member = self._mlist.members.get_member('anne@example.com')
+        self.assertIsNotNone(member)
+        items = get_queue_messages('virgin', expected_count=1)
+        self.assertEqual(str(items[0].msg['to']), 'anne@example.com')
+        self.assertEqual(
+            str(items[0].msg['subject']), 'Welcome to the "Test" mailing list')
+
 
 class CustomLayer(ConfigLayer):
     """Custom layer which starts both the REST and LMTP servers."""
@@ -707,3 +895,94 @@ class TestAPI31Members(unittest.TestCase):
         self.assertEqual(
             cm.exception.reason,
             'anne@example.com is already an owner of ant@example.com')
+
+
+class TestMemberFields(unittest.TestCase):
+    layer = RESTLayer
+
+    def setUp(self):
+        with transaction():
+            self._mlist = create_list('test@example.com')
+        self._usermanager = getUtility(IUserManager)
+
+    def _test_get_member_roster_with_specific_fields(
+            self, url, total_size, data=None):
+        if data:
+            data['fields'] = ['member_id', 'role', 'address']
+        else:
+            url = url + '?fields=member_id&fields=role&fields=address'
+        json, response = call_api(url, data=data)
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(json['total_size'], total_size)
+        entries = json['entries']
+        self.assertEqual(len(entries), total_size)
+
+        # Response will always add http_etag.
+        fields = ['address', 'http_etag', 'member_id', 'role']
+        self.assertEqual(sorted(entries[0].keys()), fields)
+
+    def _test_get_member_roster_invalid_fields(self, url, data=None):
+        if data:
+            data['fields'] = ['bogus']
+        else:
+            url = url + '?fields=bogus'
+        with self.assertRaises(HTTPError) as cm:
+            call_api(url, data=data)
+        self.assertEqual(cm.exception.code, 400)
+        self.assertIn(
+            'Unknown field "bogus" for Member resource. Allowed fields are: ',
+            cm.exception.reason
+            )
+
+    def test_get_top_level_members_with_fields(self):
+        with transaction():
+            subscribe(self._mlist, 'Anne')
+            subscribe(self._mlist, 'Bart')
+        self._test_get_member_roster_with_specific_fields(
+            'http://localhost:9001/3.1/members', total_size=2)
+        self._test_get_member_roster_invalid_fields(
+            'http://localhost:9001/3.1/members')
+
+    def test_get_members_roster_with_fields(self):
+        with transaction():
+            subscribe(self._mlist, 'Anne')
+            subscribe(self._mlist, 'Bart')
+        self._test_get_member_roster_with_specific_fields(
+            'http://localhost:9001/3.1/lists/{}/roster/member'.format(
+                self._mlist.list_id), total_size=2)
+        self._test_get_member_roster_invalid_fields(
+            'http://localhost:9001/3.1/lists/{}/roster/member'.format(
+                self._mlist.list_id))
+
+    def test_get_owners_roster_fields(self):
+        with transaction():
+            subscribe(self._mlist, 'Anne', role=MemberRole.owner)
+            subscribe(self._mlist, 'Bart', role=MemberRole.owner)
+        self._test_get_member_roster_with_specific_fields(
+            'http://localhost:9001/3.1/lists/{}/roster/owner'.format(
+                self._mlist.list_id), total_size=2)
+        self._test_get_member_roster_invalid_fields(
+            'http://localhost:9001/3.1/lists/{}/roster/owner'.format(
+                self._mlist.list_id))
+
+    def test_get_moderator_roster_fields(self):
+        with transaction():
+            subscribe(self._mlist, 'Anne', role=MemberRole.moderator)
+            subscribe(self._mlist, 'Bart', role=MemberRole.moderator)
+        self._test_get_member_roster_with_specific_fields(
+            'http://localhost:9001/3.1/lists/{}/roster/moderator'.format(
+                self._mlist.list_id), total_size=2)
+        self._test_get_member_roster_invalid_fields(
+            'http://localhost:9001/3.1/lists/{}/roster/moderator'.format(
+                self._mlist.list_id))
+
+    def test_find_members_with_fields(self):
+        with transaction():
+            subscribe(self._mlist, 'Anne')
+        self._test_get_member_roster_with_specific_fields(
+            'http://localhost:9001/3.1/members/find',
+            data={'list_id': self._mlist.list_id},
+            total_size=1)
+        self._test_get_member_roster_invalid_fields(
+            'http://localhost:9001/3.1/members/find',
+            data={'list_id': self._mlist.list_id})
