@@ -1,4 +1,4 @@
-# Copyright (C) 2012-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2012-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -97,6 +97,26 @@ Something else.
                          'Invalid Parameter "action": Accepted Values are:'
                          ' hold, reject, discard, accept, defer.')
 
+    def test_held_message_count(self):
+        # Initially, the count should be zero.
+        url = 'http://localhost:9001/3.0/lists/ant@example.com/held/count'
+        json, resp = call_api(url)
+        self.assertEqual(resp.status_code, 200)
+        self.assertEqual(json['count'], 0)
+        # Now, verify that we get the number when a held message is added.
+        with transaction():
+            hold_message(self._mlist, self._msg)
+        json, resp = call_api(url)
+        self.assertEqual(resp.status_code, 200)
+        self.assertEqual(json['count'], 1)
+        # Hold some more to see if we get the right numbers.
+        with transaction():
+            hold_message(self._mlist, self._msg)
+            hold_message(self._mlist, self._msg)
+        json, resp = call_api(url)
+        self.assertEqual(resp.status_code, 200)
+        self.assertEqual(json['count'], 3)
+
     def test_discard(self):
         # Discarding a message removes it from the moderation queue.
         with transaction():
@@ -162,6 +182,23 @@ Something else.
         self.assertEqual(json['total_size'], 1)
         self.assertEqual(json['entries'][0]['request_id'], held_id)
 
+    def test_handle_message_with_comment(self):
+        self._msg = mfs("""\
+From: anne@example.com
+To: ant@example.com
+Subject: Hello
+Message-ID: <alpha>
+
+Something else.
+""")
+        with transaction():
+            held_id = hold_message(self._mlist, self._msg)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com'
+            '/held/{}'.format(held_id),
+            dict(action='reject', comment='Because I want to.'))
+        self.assertEqual(response.status_code, 204)
+
 
 class TestSubscriptionModeration(unittest.TestCase):
     layer = RESTLayer
@@ -170,6 +207,7 @@ class TestSubscriptionModeration(unittest.TestCase):
     def setUp(self):
         with transaction():
             self._mlist = create_list('ant@example.com')
+            self._mlist.unsubscription_policy = SubscriptionPolicy.moderate
             self._registrar = ISubscriptionManager(self._mlist)
             manager = getUtility(IUserManager)
             self._anne = manager.create_address(
@@ -227,6 +265,97 @@ class TestSubscriptionModeration(unittest.TestCase):
         emails = set(entry['email'] for entry in json['entries'])
         self.assertEqual(emails, {'anne@example.com', 'bart@example.com'})
 
+    def test_list_held_requests_with_owner(self):
+        with transaction():
+            token_1, token_owner, member = self._registrar.register(self._anne)
+            # Anne's subscription request got held.
+            self.assertIsNotNone(token_1)
+            self.assertIsNone(member)
+            token_2, token_owner, member = self._registrar.register(self._bart)
+            self.assertIsNotNone(token_2)
+            self.assertIsNone(member)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com/requests'
+            '?token_owner=moderator')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(json['total_size'], 0)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com/requests'
+            '?token_owner=subscriber')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(json['total_size'], 2)
+
+    def test_list_held_requests_count(self):
+        with transaction():
+            token_1, token_owner, member = self._registrar.register(self._anne)
+            # Anne's subscription request got held.
+            self.assertIsNotNone(token_1)
+            self.assertIsNone(member)
+            token_2, token_owner, member = self._registrar.register(self._bart)
+            self.assertIsNotNone(token_2)
+            self.assertIsNone(member)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com/requests'
+            '/count')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(json['count'], 2)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com/requests'
+            '/count?token_owner=moderator')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(json['count'], 0)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com/requests'
+            '/count?token_owner=subscriber')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(json['count'], 2)
+
+    def test_list_held_unsubscription_request(self):
+        with transaction():
+            # First, subscribe Anne and then trigger an un-subscription.
+            self._mlist.subscribe(self._bart)
+            token, token_owner, member = self._registrar.unregister(self._bart)
+            # Anne's un-subscription request got held.
+            self.assertIsNotNone(token)
+            self.assertIsNotNone(member)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com/requests'
+            '?request_type=unsubscription')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(len(json['entries']), 1)
+        # Individual request can then be fetched.
+        url = 'http://localhost:9001/3.0/lists/ant@example.com/requests/{}'
+        json, response = call_api(url.format(token))
+        self.assertEqual(json['token'], token)
+        self.assertEqual(json['token_owner'], token_owner.name)
+        self.assertEqual(json['email'], 'bart@example.com')
+        self.assertEqual(json['type'], 'unsubscription')
+        # Bart should still be a Member.
+        self.assertIsNotNone(
+            self._mlist.members.get_member('bart@example.com'))
+        # Now, accept the request.
+        json, response, call_api(url.format(token), dict(
+            action='accept',
+            ))
+        self.assertEqual(response.status_code, 200)
+        # Now, the Member should be un-subscribed.
+        self.assertIsNone(
+            self._mlist.members.get_member('bart@example.com'))
+
+    def test_unsubscription_request_count(self):
+        with transaction():
+            # First, subscribe Anne and then trigger an un-subscription.
+            self._mlist.subscribe(self._bart)
+            token, token_owner, member = self._registrar.unregister(self._bart)
+            # Anne's un-subscription request got held.
+            self.assertIsNotNone(token)
+            self.assertIsNotNone(member)
+        json, response = call_api(
+            'http://localhost:9001/3.0/lists/ant@example.com/requests/count'
+            '?request_type=unsubscription')
+        self.assertEqual(response.status_code, 200)
+        self.assertEqual(json['count'], 1)
+
     def test_individual_request(self):
         # We can view an individual request.
         with transaction():
@@ -420,6 +549,33 @@ class TestSubscriptionModeration(unittest.TestCase):
                      dict(action='reject'))
         self.assertEqual(cm.exception.code, 404)
 
+    def test_reject_with_reason(self):
+        # Try to reject a request with an additional comment/reason.
+        # POST to the request to reject it.  This leaves a bounce message in
+        # the virgin queue.
+        with transaction():
+            token, token_owner, member = self._registrar.register(self._anne)
+        # Anne's subscription request got held.
+        self.assertIsNone(member)
+        # Clear out the virgin queue, which currently contains the
+        # confirmation message sent to Anne.
+        get_queue_messages('virgin')
+        url = 'http://localhost:9001/3.0/lists/ant@example.com/requests/{}'
+        reason = 'You are not authorized!'
+        json, response = call_api(url.format(token), dict(
+            action='reject',
+            reason=reason))
+        self.assertEqual(response.status_code, 204)
+        # And the rejection message to Anne is now in the virgin queue.
+        items = get_queue_messages('virgin')
+        self.assertEqual(len(items), 1)
+        message = items[0].msg
+        self.assertEqual(message['From'], 'ant-bounces@example.com')
+        self.assertEqual(message['To'], 'anne@example.com')
+        self.assertEqual(message['Subject'],
+                         'Request to mailing list "Ant" rejected')
+        self.assertTrue(reason in message.as_string())
+
     def test_hold_keeps_holding(self):
         # POST to the request to continue holding it.
         with transaction():
