@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -270,11 +270,14 @@ class TestSiteTemplates(unittest.TestCase):
                 'list:admin:notice:subscribe': '',
                 'list:admin:notice:unrecognized': '',
                 'list:admin:notice:unsubscribe': '',
+                'list:admin:notice:disable': '',
+                'list:admin:notice:removal': '',
                 'list:member:digest:footer': '',
                 'list:member:digest:header': '',
                 'list:member:digest:masthead': '',
                 'list:member:regular:footer': 'http://example.org/footer',
                 'list:member:regular:header': 'http://example.org/header',
+                'list:user:action:invite': '',
                 'list:user:action:subscribe': '',
                 'list:user:action:unsubscribe': '',
                 'list:user:notice:goodbye': 'http://example.org/goodbye',
@@ -285,6 +288,7 @@ class TestSiteTemplates(unittest.TestCase):
                 'list:user:notice:refuse': '',
                 'list:user:notice:rejected': '',
                 'list:user:notice:welcome': 'http://example.org/welcome',
+                'list:user:notice:warning': '',
                 'password': 'some password',
                 'username': 'anne.person',
                 }, method='PUT')
