@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -41,9 +41,11 @@ class TestSystemConfiguration(unittest.TestCase):
             default_language='en',
             email_commands_max_lines='10',
             filtered_messages_are_preservable='no',
+            hold_digest='no',
             html_to_plain_text_command='/usr/bin/lynx -dump $filename',
             layout='testing',
             listname_chars='[-_.0-9a-z]',
+            masthead_threshold='4',
             noreply_address='noreply',
             pending_request_life='3d',
             post_hook='',
@@ -62,8 +64,7 @@ class TestSystemConfiguration(unittest.TestCase):
         del json['http_etag']
         self.assertEqual(json, dict(
             cache_lifetime='7d',
-            org_domain_data_url=                                  # noqa: E251
-                'https://publicsuffix.org/list/public_suffix_list.dat',
+            org_domain_data_url='https://publicsuffix.org/list/public_suffix_list.dat',  # noqa: E501
             resolver_lifetime='5s',
             resolver_timeout='3s',
             self_link='http://localhost:9001/3.0/system/configuration/dmarc',
@@ -110,6 +111,7 @@ class TestSystemConfiguration(unittest.TestCase):
         json, response = call_api(url)
         self.assertIn('http_etag', json)
         self.assertEqual(sorted(json['sections']), [
+            'ARC',
             'antispam',
             'archiver.mail_archive',
             'archiver.master',
