@@ -1,4 +1,4 @@
-# Copyright (C) 2010-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2010-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -106,7 +106,7 @@ def list_of_strings_validator(values):
     # bug where an empty string ('') would be interpreted as a valid value ['']
     # to create a singleton list, instead of empty list, which in later stages
     # would create other problems.
-    if values is '':  # noqa: F632
+    if values == '':
         return []
     if not isinstance(values, (list, tuple)):
         values = [values]
@@ -181,6 +181,15 @@ def email_or_regexp_validator(value):
             ' got {}'.format(value))
 
 
+@public
+def email_validator(value):
+    """Validate the value is a valid email."""
+    if not getUtility(IEmailValidator).is_valid(value):
+        raise ValueError(
+            'Expected a valid email address, got {}'.format(value))
+    return value
+
+
 @public
 class Validator:
     """A validator of parameter input."""
