@@ -1,4 +1,4 @@
-# Copyright (C) 2014-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2014-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -141,11 +141,15 @@ A message body.
 """.format(address))
             msgdata = {}
             result = rule.check(self._mlist, msg, msgdata)
-            self.assertTrue(result, 'NonmemberModeration rule should hit')
-            self.assertIn('member_moderation_action', msgdata)
-            self.assertEqual(
-                msgdata['member_moderation_action'], action_name,
-                'Wrong action for {}: {}'.format(address, action_name))
+            if action_name == 'accept':
+                self.assertFalse(
+                    result, 'NonmemberModeration rule should miss')
+            else:
+                self.assertTrue(result, 'NonmemberModeration rule should hit')
+                self.assertIn('member_moderation_action', msgdata)
+                self.assertEqual(
+                    msgdata['member_moderation_action'], action_name,
+                    'Wrong action for {}: {}'.format(address, action_name))
 
     def test_nonmember_fallback_to_list_defaults(self):
         # https://gitlab.com/mailman/mailman/issues/189
@@ -328,3 +332,23 @@ A message body.
             'moderation_reasons': ['No sender was found in the message.'],
             'moderation_sender': 'No sender',
             })
+
+    def test_reply_to_list(self):
+        # Test a post from a member with the list posting address in Reply-To:.
+        rule = moderation.NonmemberModeration()
+        user_manager = getUtility(IUserManager)
+        anne = user_manager.create_address('anne.person@example.com')
+        user_manager.create_address('test@example.com')
+        self._mlist.subscribe(anne, MemberRole.member)
+        msg = mfs("""\
+From: anne.person@example.com
+To: test@example.com
+Reply-To: test@example.com
+Subject: A test message
+Message-ID: <ant>
+MIME-Version: 1.0
+
+A message body.
+""")
+        result = rule.check(self._mlist, msg, {})
+        self.assertFalse(result)
