@@ -1,4 +1,4 @@
-# Copyright (C) 1998-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 1998-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -69,7 +69,9 @@ class CommandFinder:
             is_address_command = True
             self.send_response = False
         elif subaddress == 'confirm':
-            mo = re.match(config.mta.verp_confirm_regexp, msg.get('to', ''))
+            # match with re.DOTALL in case header is folded.
+            mo = re.match(config.mta.verp_confirm_regexp,
+                          msg.get('to', ''), re.DOTALL)
             if mo:
                 self.command_lines.append('confirm ' + mo.group('cookie'))
                 is_address_command = True
