@@ -1,4 +1,4 @@
-# Copyright (C) 2016-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2016-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -82,7 +82,7 @@ leave
         # sent.
         items = get_queue_messages('virgin', expected_count=1)
         confirmation = items[0].msg
-        self.assertTrue(str(confirmation['subject']).startswith('confirm'))
+        self.assertTrue(str(confirmation['subject']).startswith('Your conf'))
 
     def test_double_leave(self):
         # In this case, the user can be unsubscribed immediately because the
