@@ -1,4 +1,4 @@
-# Copyright (C) 2008-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2008-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -30,7 +30,7 @@ import datetime
 import threading
 
 from contextlib import contextmanager, suppress
-from email import message_from_string
+from email import message_from_bytes, message_from_string
 from lazr.config import as_timedelta
 from mailman.bin.master import Loop as Master
 from mailman.config import config
@@ -83,6 +83,7 @@ def make_testable_runner(runner_class, name=None, predicate=None):
 
         def _do_periodic(self):
             """Stop when the queue is empty."""
+            super()._do_periodic()
             if predicate is None:
                 self._stop = (len(self.switchboard.files) == 0)
             else:
@@ -250,7 +251,7 @@ def get_nntp_server(cleanups):
     class NNTPProxy:                                              # noqa: E306
         def get_message(self):
             args = nntpd.post.call_args
-            return specialized_message_from_string(args[0][0].read())
+            return message_from_bytes(args[0][0].read())
     return NNTPProxy()
 
 
@@ -416,7 +417,8 @@ class chdir:
 
 
 @public
-def subscribe(mlist, first_name, role=MemberRole.member, email=None):
+def subscribe(mlist, first_name, role=MemberRole.member, email=None,
+              as_user=False):
     """Helper for subscribing a sample person to a mailing list.
 
     Returns the newly created member object.
@@ -431,12 +433,27 @@ def subscribe(mlist, first_name, role=MemberRole.member, email=None):
             address = user_manager.get_address(email)
             if address is None:
                 person = user_manager.create_user(email, full_name)
-                subscription_address = list(person.addresses)[0]
+                set_preferred(person)
+                if as_user:
+                    subscription_object = person
+                else:
+                    subscription_object = person.preferred_address
             else:
-                subscription_address = address
+                if not as_user:
+                    subscription_object = address
+                else:
+                    person = user_manager.create_user(display_name=full_name)
+                    person.link(address)
+                    set_preferred(person)
+                    subscription_object = person
         else:
-            subscription_address = list(person.addresses)[0]
-        mlist.subscribe(subscription_address, role)
+            if as_user:
+                if person.preferred_address is None:
+                    set_preferred(person)
+                subscription_object = person
+            else:
+                subscription_object = list(person.addresses)[0]
+        mlist.subscribe(subscription_object, role)
         roster = mlist.get_roster(role)
         return roster.get_member(email)
 
