@@ -1,4 +1,4 @@
-# Copyright (C) 2011-2019 by the Free Software Foundation, Inc.
+# Copyright (C) 2011-2021 by the Free Software Foundation, Inc.
 #
 # This file is part of GNU Mailman.
 #
@@ -23,11 +23,11 @@ import tempfile
 import unittest
 
 from contextlib import ExitStack
-from importlib_resources import path as resource_path
 from mailman.app.lifecycle import create_list
 from mailman.config import config
 from mailman.interfaces.languages import ILanguageManager
 from mailman.testing.layers import ConfigLayer
+from mailman.utilities.filesystem import path as resource_path
 from mailman.utilities.i18n import TemplateNotFoundError, find, search
 from zope.component import getUtility
 
